@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,6 +22,7 @@
 # This file ensures that renpy packages will be imported in the right
 # order.
 
+from __future__ import print_function
 import sys
 import os
 import copy
@@ -40,10 +41,10 @@ except ImportError:
     vc_version = 0
 
 # The tuple giving the version number.
-version_tuple = (6, 99, 7, vc_version)
+version_tuple = (7, 0, 0, vc_version)
 
 # The name of this version.
-version_name = "Here's to the crazy ones."
+version_name = "For all mankind."
 
 # A string giving the version number only (7.0.1.123).
 version_only = ".".join(str(i) for i in version_tuple)
@@ -70,10 +71,50 @@ linux = False
 android = False
 ios = False
 
+# Should we enable experimental features and debugging?
+experimental = "RENPY_EXPERIMENTAL" in os.environ
+
 import platform
 
+
+def get_windows_version():
+    """
+    When called on windows, returns the windows version.
+    """
+
+    import ctypes
+
+    class OSVERSIONINFOEXW(ctypes.Structure):
+        _fields_ = [('dwOSVersionInfoSize', ctypes.c_ulong),
+                    ('dwMajorVersion', ctypes.c_ulong),
+                    ('dwMinorVersion', ctypes.c_ulong),
+                    ('dwBuildNumber', ctypes.c_ulong),
+                    ('dwPlatformId', ctypes.c_ulong),
+                    ('szCSDVersion', ctypes.c_wchar*128),
+                    ('wServicePackMajor', ctypes.c_ushort),
+                    ('wServicePackMinor', ctypes.c_ushort),
+                    ('wSuiteMask', ctypes.c_ushort),
+                    ('wProductType', ctypes.c_byte),
+                    ('wReserved', ctypes.c_byte)]
+
+    try:
+
+        os_version = OSVERSIONINFOEXW()
+        os_version.dwOSVersionInfoSize = ctypes.sizeof(os_version)
+        retcode = ctypes.windll.Ntdll.RtlGetVersion(ctypes.byref(os_version))
+
+        # Om failure, assume we have a newer version of windows
+        if retcode != 0:
+            return (10, 0)
+
+        return (os_version.dwMajorVersion, os_version.dwMinorVersion)
+
+    except:
+        return (10, 0)
+
+
 if platform.win32_ver()[0]:
-    windows = True
+    windows = get_windows_version()
 elif "RENPY_IOS" in os.environ:
     ios = True
 elif platform.mac_ver()[0]:
@@ -86,6 +127,9 @@ else:
 # A flag that's true if we're on a smartphone or tablet-like platform.
 mobile = android or ios
 
+# A flag that's set to true if the game directory is bundled inside a mac app.
+macapp = False
+
 
 ################################################################################
 # Backup Data for Reload
@@ -110,11 +154,20 @@ backup_blacklist = {
     "renpy.object",
     "renpy.log",
     "renpy.bootstrap",
+    "renpy.debug",
     "renpy.display",
     "renpy.display.pgrender",
     "renpy.display.scale",
     "renpy.display.presplash",
+    "renpy.display.test",
     "renpy.text.ftfont",
+    "renpy.test",
+    "renpy.test.testast",
+    "renpy.test.testexecution",
+    "renpy.test.testkey",
+    "renpy.test.testmouse",
+    "renpy.test.testparser",
+    "renpycoverage",
     }
 
 type_blacklist = (
@@ -137,8 +190,11 @@ name_blacklist = {
     "renpy.display.render.IDENTITY",
     "renpy.loader.auto_lock",
     "renpy.display.screen.cprof",
+    "renpy.audio.audio.lock",
+    "renpy.audio.audio.periodic_condition",
     }
 
+
 class Backup():
     """
     This represents a backup of all of the fields in the python modules
@@ -180,7 +236,10 @@ class Backup():
         Makes a backup of `mod`, which must be a Python module.
         """
 
-        name = mod.__name__
+        try:
+            name = mod.__name__
+        except:
+            return
 
         if not name.startswith("renpy"):
             return
@@ -214,8 +273,8 @@ class Backup():
             try:
                 cPickle.dumps(v, cPickle.HIGHEST_PROTOCOL)
             except:
-                print "Cannot pickle", name + "." + k, "=", repr(v)
-                print "Reduce Ex is:", repr(v.__reduce_ex__(cPickle.HIGHEST_PROTOCOL))
+                print("Cannot pickle", name + "." + k, "=", repr(v))
+                print("Reduce Ex is:", repr(v.__reduce_ex__(cPickle.HIGHEST_PROTOCOL)))
 
     def restore(self):
         """
@@ -232,13 +291,13 @@ class Backup():
             for name in set(modvars.keys()) - names:
                 del modvars[name]
 
-
         objects = cPickle.loads(self.objects_pickle)
 
         for k, v in self.variables.iteritems():
             mod, field = k
             setattr(mod, field, objects[v])
 
+
 # A backup of the Ren'Py modules after initial import.
 backup = None
 
@@ -246,6 +305,7 @@ backup = None
 # Import
 ################################################################################
 
+
 def update_path(package):
     """
     Update the __path__ of package, to import binary modules from a libexec
@@ -263,21 +323,33 @@ def update_path(package):
     libexec = os.path.dirname(encodings.__path__[0])
     package.__path__.append(os.path.join(libexec, *name))
 
+# Replaced below.
+
+
+def plog(level, even, *args):
+    return
+
+
 def import_all():
 
     # Note: If we add a new update_path, we have to add an equivalent
     # hook in the renpython hooks dir.
 
-    import renpy # @UnresolvedImport
+    # Note: If we add a new module, we need to add it to iOS.
+
+    import renpy  # @UnresolvedImport
 
     update_path(renpy)
 
-    import renpy.arguments # @UnresolvedImport
+    import renpy.arguments  # @UnresolvedImport
 
+    import renpy.config
     import renpy.log
 
     import renpy.display
 
+    import renpy.debug
+
     # Should probably be early, as we will add it as a base to serialized things.
     import renpy.object
 
@@ -300,11 +372,16 @@ def import_all():
     import renpy.persistent
     import renpy.scriptedit
     import renpy.parser
+    import renpy.performance
+    import renpy.pydict
     import renpy.python
     import renpy.script
     import renpy.statements
 
-    import renpy.styledata # @UnresolvedImport
+    global plog
+    plog = renpy.performance.log
+
+    import renpy.styledata  # @UnresolvedImport
     update_path(renpy.styledata)
 
     import renpy.style
@@ -314,8 +391,13 @@ def import_all():
 
     import renpy.substitutions
     import renpy.translation
+    import renpy.translation.scanstrings
+    import renpy.translation.generation
+    import renpy.translation.dialogue
+    import renpy.translation.extract
+    import renpy.translation.merge
 
-    import renpy.display # @UnresolvedImport @Reimport
+    import renpy.display  # @UnresolvedImport @Reimport
 
     update_path(renpy.display)
 
@@ -323,8 +405,8 @@ def import_all():
     import renpy.display.pgrender
     import renpy.display.scale
     import renpy.display.module
-    import renpy.display.render # Most display stuff depends on this. @UnresolvedImport
-    import renpy.display.core # object @UnresolvedImport
+    import renpy.display.render  # Most display stuff depends on this. @UnresolvedImport
+    import renpy.display.core  # object @UnresolvedImport
 
     import renpy.text
 
@@ -346,13 +428,15 @@ def import_all():
     update_path(renpy.angle)
 
     import renpy.display.layout
-    import renpy.display.motion # layout @UnresolvedImport
-    import renpy.display.behavior # layout @UnresolvedImport
-    import renpy.display.transition # core, layout @UnresolvedImport
-    import renpy.display.movetransition # core @UnresolvedImport
+    import renpy.display.viewport
+    import renpy.display.transform
+    import renpy.display.motion  # layout @UnresolvedImport
+    import renpy.display.behavior  # layout @UnresolvedImport
+    import renpy.display.transition  # core, layout @UnresolvedImport
+    import renpy.display.movetransition  # core @UnresolvedImport
     import renpy.display.im
     import renpy.display.imagelike
-    import renpy.display.image # core, behavior, im, imagelike @UnresolvedImport
+    import renpy.display.image  # core, behavior, im, imagelike @UnresolvedImport
     import renpy.display.video
     import renpy.display.focus
     import renpy.display.anim
@@ -372,6 +456,10 @@ def import_all():
 
     # Note: For windows to work, renpy.audio.audio needs to be after
     # renpy.display.module.
+
+    import renpy.audio
+    update_path(renpy.audio)
+
     import renpy.audio.audio
     import renpy.audio.music
     import renpy.audio.sound
@@ -395,17 +483,23 @@ def import_all():
     import renpy.memory
 
     import renpy.exports
-    import renpy.character # depends on exports. @UnresolvedImport
+    import renpy.character  # depends on exports. @UnresolvedImport
 
     import renpy.add_from
     import renpy.dump
 
-    import renpy.config # depends on lots. @UnresolvedImport
-    import renpy.minstore # depends on lots. @UnresolvedImport
+    import renpy.minstore  # depends on lots. @UnresolvedImport
     import renpy.defaultstore  # depends on everything. @UnresolvedImport
 
-    import renpy.main
+    import renpy.test
+    import renpy.test.testmouse
+    import renpy.test.testfocus
+    import renpy.test.testkey
+    import renpy.test.testast
+    import renpy.test.testparser
+    import renpy.test.testexecution
 
+    import renpy.main
 
     # Back up the Ren'Py modules.
 
@@ -423,7 +517,7 @@ def post_import():
     of various modules.
     """
 
-    import renpy # @UnresolvedImport
+    import renpy  # @UnresolvedImport
 
     # Create the store.
     renpy.python.create_store("store")
@@ -440,6 +534,8 @@ def post_import():
     for k, v in renpy.defaultstore.__dict__.iteritems():
         renpy.store.__dict__.setdefault(k, v)
 
+    renpy.store.eval = renpy.defaultstore.eval
+
     # Import everything into renpy.exports, provided it isn't
     # already there.
     for k, v in globals().iteritems():
@@ -462,7 +558,7 @@ def reload_all():
     sys.exc_clear()
 
     # Reset the styles.
-    renpy.style.reset() # @UndefinedVariable
+    renpy.style.reset()  # @UndefinedVariable
 
     # Shut down the cache thread.
     renpy.display.im.cache.quit()
@@ -482,6 +578,8 @@ def reload_all():
     renpy.display.draw = None
     renpy.display.interface = None
 
+    py_compile_cache = renpy.python.py_compile_cache
+
     # Delete the store modules.
     for i in sys.modules.keys():
         if i.startswith("store") or i == "renpy.store":
@@ -495,6 +593,8 @@ def reload_all():
     # Restore the state of all modules from backup.
     backup.restore()
 
+    renpy.python.old_py_compile_cache = py_compile_cache
+
     renpy.display.im.reset_module()
 
     post_import()
@@ -553,8 +653,5 @@ def import_cython():
     import renpy.angle.gltexture
 
 
-
 if False:
     import renpy.defaultstore as store
-
-
