@@ -72,6 +72,7 @@ from renpy.display.predict import screen as predict_screen
 
 from renpy.display.image import image_exists, image_exists as has_image, list_images
 from renpy.display.image import get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
+from renpy.display.image import get_registered_image
 
 from renpy.display.im import load_surface, load_image
 
@@ -165,6 +166,7 @@ def public_api():
     const, pure, not_const
     image_exists, has_image, list_images
     get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
+    get_registered_image
     load_image, load_surface
     profile_memory, diff_memory, profile_rollback
     TEXT_TAG
@@ -269,15 +271,16 @@ def checkpoint(data=None, keep_rollback=None):
     renpy.game.log.checkpoint(data, keep_rollback=keep_rollback, hard=renpy.store._rollback)
 
 
-def block_rollback():
+def block_rollback(purge=False):
     """
     :doc: blockrollback
+    :args: ()
 
     Prevents the game from rolling back to before the current
     statement.
     """
 
-    renpy.game.log.block()
+    renpy.game.log.block(purge=purge)
 
 
 def suspend_rollback(flag):
@@ -479,22 +482,38 @@ def showing(name, layer='master'):
     return renpy.game.context().images.showing(layer, name)
 
 
-def get_showing_tags(layer='master'):
+def get_showing_tags(layer='master', sort=False):
     """
     :doc: image_func
 
-    Returns the set of image tags that are currently being shown on `layer`
+    Returns the set of image tags that are currently being shown on `layer`. If
+    sort is true, returns a list of the tags from back to front.
     """
 
+    if sort:
+        return scene_lists().get_sorted_tags(layer)
+
     return renpy.game.context().images.get_showing_tags(layer)
 
 
+def get_hidden_tags(layer='master'):
+    """
+    :doc: image_func
+
+    Returns the set of image tags on `layer` that are currently hidden, but
+    still have attribute information associated with them.
+    """
+
+    return renpy.game.context().images.get_hidden_tags(layer)
+
+
 def get_attributes(tag, layer=None):
     """
     :doc: image_func
 
-    Return a tuple giving the image attributes for the image with `tag`. If
-    the image is not showing, returns None.
+    Return a tuple giving the image attributes for the image `tag`. If
+    the image tag has not had any attributes associated since the last
+    time it was hidden, returns None.
 
     `layer`
         The layer to check. If None, uses the default layer for `tag`.
@@ -1410,15 +1429,36 @@ def with_statement(trans, always=False, paired=None, clear=True):
 
     renpy.exports.mode('with')
 
-    return renpy.game.interface.do_with(trans, paired, clear=clear)
+    if isinstance(paired, dict):
+        paired = paired.get(None, None)
+
+        if (trans is None) and (paired is None):
+            return
+
+    if isinstance(trans, dict):
+
+        for k, v in trans.items():
+            if k is None:
+                continue
+
+            renpy.exports.transition(v, layer=k)
+
+        if None not in trans:
+            return
+
+        trans = trans[None]
+
+    else:
+        return renpy.game.interface.do_with(trans, paired, clear=clear)
 
 
 globals()["with"] = with_statement
 
 
-def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None, abnormal=True):
+def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None, abnormal=True, current_label=None):
     """
     :doc: rollback
+    :args: (force=False, checkpoints=1, defer=False, greedy=True, label=None, abnormal=True)
 
     Rolls the state of the game back to the last checkpoint.
 
@@ -1465,7 +1505,7 @@ def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None, a
 
     renpy.config.skipping = None
     renpy.game.log.complete()
-    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=(force is True), abnormal=abnormal)
+    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=(force is True), abnormal=abnormal, current_label=current_label)
 
 
 def toggle_fullscreen():
@@ -1720,6 +1760,11 @@ def transition(trans, layer=None, always=False, force=False):
         transition is always run.
     """
 
+    if isinstance(trans, dict):
+        for k, v in trans.items():
+            trans(k, v, always=always, force=force)
+        return
+
     if not always and not renpy.game.preferences.transitions:
         trans = None
 
@@ -2431,13 +2476,14 @@ def get_placement(d):
     This returns an object with the following fields, each corresponding to a style
     property:
 
-        * xpos
-        * xanchor
-        * xoffset
-        * ypos
-        * yanchor
-        * yoffset
-        * subpixel
+    * xpos
+    * xanchor
+    * xoffset
+    * ypos
+    * yanchor
+    * yoffset
+    * subpixel
+
     """
     p = d.get_placement()
 
