@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -83,6 +83,12 @@ def get_store_module(name):
     return sys.modules[name]
 
 
+from renpy.pydict import DictItems, find_changes
+
+EMPTY_DICT = { }
+EMPTY_SET = set()
+
+
 class StoreDict(dict):
     """
     This class represents the dictionary of a store module. It logs
@@ -96,7 +102,7 @@ class StoreDict(dict):
 
         # The value of this dictionary at the start of the current
         # rollback period (when begin() was last called).
-        self.old = { }
+        self.old = DictItems(self)
 
         # The set of variables in this StoreDict that changed since the
         # end of the init phase.
@@ -107,43 +113,59 @@ class StoreDict(dict):
         Called to reset this to its initial conditions.
         """
 
-        self.old = { }
         self.ever_been_changed = set()
         self.clear()
+        self.old = DictItems(self)
 
     def begin(self):
         """
         Called to mark the start of a rollback period.
         """
 
-        self.old = dict(self)
+        self.old = DictItems(self)
 
-    def get_changes(self):
+    def get_changes(self, cycle):
         """
         For every key that has changed since begin() was called, returns a
         dictionary mapping the key to its value when begin was called, or
         deleted if it did not exist when begin was called.
 
-        As a side-effect, updates self.ever_been_changed.
+        As a side-effect, updates self.ever_been_changed, and returns the
+        changes to ever_been_changed as well.
+
+        `cycle`
+            If true, this cycles the old changes to the new changes. If
+            False, does not.
         """
 
-        rv = { }
+        new = DictItems(self)
+        rv = find_changes(self.old, new, deleted)
 
-        for k in self:
-            if k not in self.old:
-                rv[k] = deleted
+        if cycle:
+            self.old = new
 
-        for k, v in self.old.iteritems():
+        if rv is None:
+            return EMPTY_DICT, EMPTY_SET
 
-            new_v = self.get(k, deleted)
+        delta_ebc = set()
 
-            if new_v is not v:
-                rv[k] = v
+        if cycle:
 
-        for k in rv:
-            self.ever_been_changed.add(k)
+            for k in rv:
+                if k not in self.ever_been_changed:
+                    self.ever_been_changed.add(k)
+                    delta_ebc.add(k)
 
-        return rv
+        return rv, delta_ebc
+
+
+def begin_stores():
+    """
+    Calls .begin on every store dict.
+    """
+
+    for sd in store_dicts.itervalues():
+        sd.begin()
 
 
 # A map from the name of a store dict to the corresponding StoreDict object.
@@ -217,10 +239,16 @@ class StoreBackup():
         # The contents of ever_been_changed for each store.
         self.ever_been_changed = { }
 
-        for k, v in store_dicts.iteritems():
-            self.store[k] = dict(v)
-            self.old[k] = dict(v.old)
-            self.ever_been_changed[k] = set(v.ever_been_changed)
+        for k in store_dicts:
+            self.backup_one(k)
+
+    def backup_one(self, name):
+
+        d = store_dicts[name]
+
+        self.store[name] = dict(d)
+        self.old[name] = d.old.as_dict()
+        self.ever_been_changed[name] = set(d.ever_been_changed)
 
     def restore_one(self, name):
         sd = store_dicts[name]
@@ -228,8 +256,7 @@ class StoreBackup():
         sd.clear()
         sd.update(self.store[name])
 
-        sd.old.clear()
-        sd.old.update(self.old[name])
+        sd.old = DictItems(self.old[name])
 
         sd.ever_been_changed.clear()
         sd.ever_been_changed.update(self.ever_been_changed[name])
@@ -252,8 +279,8 @@ def make_clean_stores():
 
     for _k, v in store_dicts.iteritems():
 
-        v.old.clear()
         v.ever_been_changed.clear()
+        v.begin()
 
     clean_store_backup = StoreBackup()
 
@@ -468,18 +495,23 @@ class WrapNode(ast.NodeTransformer):
 wrap_node = WrapNode()
 
 
-def set_filename(filename, offset, tree):
-    """Set the filename attribute to filename on every node in tree"""
-    worklist = [tree]
-    while worklist:
-        node = worklist.pop(0)
-        node.filename = filename
+def wrap_hide(tree):
+    """
+    Wraps code inside a python hide or python early hide block inside a
+    function, so it gets its own scope that works the way Python expects
+    it to.
+    """
 
-        lineno = getattr(node, 'lineno', None)
-        if lineno is not None:
-            node.lineno = lineno + offset
+    hide = ast.parse("""\
+def _execute_python_hide(): pass;
+_execute_python_hide()
+""")
 
-        worklist.extend(node.getChildNodes())
+    for i in ast.walk(hide):
+        ast.copy_location(i, hide.body[0])
+
+    hide.body[0].body = tree.body
+    tree.body = hide.body
 
 
 unicode_re = re.compile(ur'[\u0080-\uffff]')
@@ -530,7 +562,14 @@ new_compile_flags = (  old_compile_flags
                        )
 
 
-def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False):
+# A cache for the results of py_compile.
+py_compile_cache = { }
+
+# An old version of the same, that's preserved across reloads.
+old_py_compile_cache = { }
+
+
+def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False, cache=True):
     """
     Compiles the given source code using the supplied codegenerator.
     Lists, List Comprehensions, and Dictionaries are wrapped when
@@ -556,6 +595,9 @@ def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False):
         that would be used.
     """
 
+    if ast_node:
+        cache = False
+
     if isinstance(source, ast.Module):
         return compile(source, filename, mode)
 
@@ -563,6 +605,26 @@ def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False):
         filename = source.filename
         lineno = source.linenumber
 
+    if cache:
+        key = (lineno, filename, unicode(source), mode, renpy.script.MAGIC)
+
+        rv = py_compile_cache.get(key, None)
+        if rv is not None:
+            return rv
+
+        rv = old_py_compile_cache.get(key, None)
+        if rv is not None:
+            old_py_compile_cache[key] = rv
+            return rv
+
+        bytecode = renpy.game.script.bytecode_oldcache.get(key, None)
+        if bytecode is not None:
+
+            renpy.game.script.bytecode_newcache[key] = bytecode
+            rv = marshal.loads(bytecode)
+            py_compile_cache[key] = rv
+            return rv
+
     source = unicode(source)
     source = source.replace("\r", "")
     source = escape_unicode(source)
@@ -570,15 +632,23 @@ def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False):
     try:
         line_offset = lineno - 1
 
+        if mode == "hide":
+            py_mode = "exec"
+        else:
+            py_mode = mode
+
         try:
             flags = new_compile_flags
-            tree = compile(source, filename, mode, ast.PyCF_ONLY_AST | flags, 1)
+            tree = compile(source, filename, py_mode, ast.PyCF_ONLY_AST | flags, 1)
         except:
             flags = old_compile_flags
-            tree = compile(source, filename, mode, ast.PyCF_ONLY_AST | flags, 1)
+            tree = compile(source, filename, py_mode, ast.PyCF_ONLY_AST | flags, 1)
 
         tree = wrap_node.visit(tree)
 
+        if mode == "hide":
+            wrap_hide(tree)
+
         ast.fix_missing_locations(tree)
         ast.increment_lineno(tree, lineno - 1)
 
@@ -587,7 +657,14 @@ def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False):
         if ast_node:
             return tree.body
 
-        return compile(tree, filename, mode, flags, 1)
+        rv = compile(tree, filename, py_mode, flags, 1)
+
+        if cache:
+            py_compile_cache[key] = rv
+            renpy.game.script.bytecode_newcache[key] = marshal.dumps(rv)
+            renpy.game.script.bytecode_dirty = True
+
+        return rv
 
     except SyntaxError, e:
 
@@ -598,13 +675,18 @@ def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False):
 
 
 def py_compile_exec_bytecode(source, **kwargs):
-    code = py_compile(source, 'exec', **kwargs)
+    code = py_compile(source, 'exec', cache=False, **kwargs)
+    return marshal.dumps(code)
+
+
+def py_compile_hide_bytecode(source, **kwargs):
+    code = py_compile(source, 'hide', cache=False, **kwargs)
     return marshal.dumps(code)
 
 
 def py_compile_eval_bytecode(source, **kwargs):
     source = source.strip()
-    code = py_compile(source, 'eval', **kwargs)
+    code = py_compile(source, 'eval', cache=False, **kwargs)
     return marshal.dumps(code)
 
 
@@ -690,10 +772,10 @@ class CompressedList(object):
             old_end += 1
 
         # Now that we have this, we can put together the object.
-        self.pre = old[0:old_start]
+        self.pre = list.__getslice__(old, 0, old_start)
         self.start = new_start
         self.end = new_end
-        self.post = old[old_end:]
+        self.post = list.__getslice__(old, old_end, len_old)
 
     def decompress(self, new):
         return self.pre + new[self.start:self.end] + self.post
@@ -1036,7 +1118,7 @@ class Rollback(renpy.object.Object):
     execution of this element.
     """
 
-    __version__ = 4
+    __version__ = 5
 
     identifier = None
 
@@ -1045,6 +1127,7 @@ class Rollback(renpy.object.Object):
         super(Rollback, self).__init__()
 
         self.context = renpy.game.context().rollback_copy()
+
         self.objects = [ ]
         self.purged = False
         self.random = [ ]
@@ -1053,6 +1136,10 @@ class Rollback(renpy.object.Object):
         # A map of maps name -> (variable -> value)
         self.stores = { }
 
+        # A map from store name to the changes to ever_been_changed that
+        # need to be reverted.
+        self.delta_ebc = { }
+
         # If true, we retain the data in this rollback when a load occurs.
         self.retain_after_load = False
 
@@ -1088,6 +1175,9 @@ class Rollback(renpy.object.Object):
         if version < 4:
             self.hard_checkpoint = self.checkpoint
 
+        if version < 5:
+            self.delta_ebc = { }
+
     def purge_unreachable(self, reachable, wait):
         """
         Adds objects that are reachable from the store of this
@@ -1128,8 +1218,7 @@ class Rollback(renpy.object.Object):
                 reached(rb, reachable, wait)
             else:
                 if renpy.config.debug:
-                    print(("Removing unreachable:", o))
-
+                    print("Removing unreachable:", o, file=renpy.log.real_stdout)
                     pass
 
         self.objects = new_objects
@@ -1149,7 +1238,7 @@ class Rollback(renpy.object.Object):
         for name, changes in self.stores.iteritems():
             store = store_dicts.get(name, None)
             if store is None:
-                return
+                continue
 
             for name, value in changes.iteritems():
                 if value is deleted:
@@ -1158,6 +1247,14 @@ class Rollback(renpy.object.Object):
                 else:
                     store[name] = value
 
+        for name, changes in self.delta_ebc.iteritems():
+
+            store = store_dicts.get(name, None)
+            if store is None:
+                continue
+
+            store.ever_been_changed -= changes
+
         rng.pushback(self.random)
 
         renpy.game.contexts.pop()
@@ -1220,6 +1317,10 @@ class RollbackLog(renpy.object.Object):
         # on load.
         self.retain_after_load_flag = False
 
+        # Has there been an interaction since the last time this log was
+        # reset?
+        self.did_interaction = True
+
     def after_setstate(self):
         self.mutated = { }
         self.rolled_forward = False
@@ -1246,7 +1347,7 @@ class RollbackLog(renpy.object.Object):
 
                 self.rollback_limit = nrbl
 
-    def begin(self):
+    def begin(self, force=False):
         """
         Called before a node begins executing, to indicate that the
         state needs to be saved for rollbacking.
@@ -1259,12 +1360,30 @@ class RollbackLog(renpy.object.Object):
         if not context.rollback:
             return
 
-        # If the transient scene list is not empty, then we do
-        # not begin a new rollback, as the TSL will be purged
-        # after a rollback is complete.
-        if not context.scene_lists.transient_is_empty():
+        # We only begin a checkpoint if the previous statement reached a checkpoint,
+        # or an interaction took place. (Or we're forced.)
+        ignore = True
+
+        if force:
+            ignore = False
+        elif self.did_interaction:
+            ignore = False
+        elif self.current is not None:
+            if self.current.checkpoint:
+                ignore = False
+            elif self.current.retain_after_load:
+                ignore = False
+
+        if ignore:
             return
 
+        self.did_interaction = False
+
+        if self.current is not None:
+            self.complete(True)
+        else:
+            begin_stores()
+
         # If the log is too long, prune it.
         if len(self.log) > renpy.config.rollback_length:
             self.log = self.log[-renpy.config.rollback_length:]
@@ -1294,23 +1413,31 @@ class RollbackLog(renpy.object.Object):
 
         self.rolled_forward = False
 
-        # Reset the point that changes are relative to.
-        for sd in store_dicts.itervalues():
-            sd.begin()
+    def replace_node(self, old, new):
+        """
+        Replaces references to the `old` ast node with a reference to the
+        `new` ast node.
+        """
 
-    def complete(self):
+        for i in self.log:
+            i.context.replace_node(old, new)
+
+    def complete(self, begin=False):
         """
         Called after a node is finished executing, before a save
         begins, or right before a rollback is attempted. This may be
         called more than once between calls to begin, and should always
         be called after an update to the store but before a rollback
         occurs.
+
+        `begin`
+            Should be true if called from begin().
         """
 
         # Update self.current.stores with the changes from each store.
         # Also updates .ever_been_changed.
         for name, sd in store_dicts.iteritems():
-            self.current.stores[name] = sd.get_changes()
+            self.current.stores[name], self.current.delta_ebc[name] = sd.get_changes(begin)
 
         # Update the list of mutated objects and what we need to do to
         # restore them.
@@ -1448,8 +1575,8 @@ class RollbackLog(renpy.object.Object):
                 fwd_name, fwd_data = self.forward[0]
 
                 if (self.current.context.current == fwd_name
-                        and data == fwd_data
-                        and (keep_rollback or self.rolled_forward)
+                            and data == fwd_data
+                            and (keep_rollback or self.rolled_forward)
                         ):
                     self.forward.pop(0)
                 else:
@@ -1466,13 +1593,17 @@ class RollbackLog(renpy.object.Object):
 
         self.checkpointing_suspended = flag
 
-    def block(self):
+    def block(self, purge=False):
         """
         Called to indicate that the user should not be able to rollback
         through this checkpoint.
         """
 
         self.rollback_limit = 0
+        renpy.game.context().force_checkpoint = True
+
+        if purge:
+            self.log = [ ]
 
     def retain_after_load(self):
         """
@@ -1480,8 +1611,12 @@ class RollbackLog(renpy.object.Object):
         when the game is loaded.
         """
 
+        if renpy.display.predict.predicting:
+            return
+
         self.retain_after_load_flag = True
         self.current.retain_after_load = True
+        renpy.game.context().force_checkpoint = True
 
     def fix_rollback(self):
         if not self.rollback_is_fixed and len(self.log) > 1:
@@ -1494,7 +1629,7 @@ class RollbackLog(renpy.object.Object):
 
         return self.rollback_limit > 0
 
-    def rollback(self, checkpoints, force=False, label=None, greedy=True, on_load=False, abnormal=True):
+    def rollback(self, checkpoints, force=False, label=None, greedy=True, on_load=False, abnormal=True, current_label=None):
         """
         This rolls the system back to the first valid rollback point
         after having rolled back past the specified number of checkpoints.
@@ -1522,6 +1657,11 @@ class RollbackLog(renpy.object.Object):
         `abnormal`
             If true, treats this as an abnormal event, suppresisng rollback
             and so on.
+
+        `current_label`
+            A lable that is called when control returns to the current statement,
+            after rollback. (At most one of `current_label` and `label` can be
+            provided.)
         """
 
         # If we have exceeded the rollback limit, and don't have force,
@@ -1566,21 +1706,14 @@ class RollbackLog(renpy.object.Object):
         force_checkpoint = False
 
         # Try to rollback to just after the previous checkpoint.
-        while greedy and self.log:
+        while greedy and self.log and (self.rollback_limit > 0):
 
             rb = self.log[-1]
 
             if not renpy.game.script.has_label(rb.context.current):
                 break
 
-            if rb.checkpoint:
-
-                # If the last checkpoint is a label, it's likely the start label.
-                # go back to it.
-                if isinstance(rb.context.current, basestring):
-                    force_checkpoint = True
-                    revlog.append(self.log.pop())
-
+            if rb.hard_checkpoint:
                 break
 
             revlog.append(self.log.pop())
@@ -1590,6 +1723,7 @@ class RollbackLog(renpy.object.Object):
         if renpy.game.context().rollback:
             replace_context = False
             other_contexts = [ ]
+
         else:
             replace_context = True
             other_contexts = renpy.game.contexts[1:]
@@ -1600,6 +1734,12 @@ class RollbackLog(renpy.object.Object):
         else:
             retained = None
 
+        come_from = None
+
+        if current_label is not None:
+            come_from = renpy.game.context().current
+            label = current_label
+
         # Actually roll things back.
         for rb in revlog:
             rb.rollback()
@@ -1614,6 +1754,12 @@ class RollbackLog(renpy.object.Object):
             retained.rollback_control()
             self.log.append(retained)
 
+        if (label is not None) and (come_from is None):
+            come_from = renpy.game.context().current
+
+        if come_from is not None:
+            renpy.game.context().come_from(come_from, label)
+
         # Disable the next transition, as it's pointless. (Only when not used with a label.)
         renpy.game.interface.suppress_transition = abnormal
 
@@ -1634,20 +1780,22 @@ class RollbackLog(renpy.object.Object):
 
         renpy.game.contexts.extend(other_contexts)
 
+        begin_stores()
+
         # Restart the context or the top context.
         if replace_context:
 
             if force_checkpoint:
                 renpy.game.contexts[0].force_checkpoint = True
 
-            raise renpy.game.RestartTopContext(label)
+            raise renpy.game.RestartTopContext()
 
         else:
 
             if force_checkpoint:
                 renpy.game.context().force_checkpoint = True
 
-            raise renpy.game.RestartContext(label)
+            raise renpy.game.RestartContext()
 
     def freeze(self, wait=None):
         """
@@ -1658,7 +1806,7 @@ class RollbackLog(renpy.object.Object):
         """
 
         # Purge unreachable objects, so we don't save them.
-        self.complete()
+        self.complete(False)
         roots = self.get_roots()
         self.purge_unreachable(roots, wait=wait)
 
@@ -1791,9 +1939,18 @@ def py_eval_bytecode(bytecode, globals=None, locals=None):  # @ReservedAssignmen
 def py_eval(code, globals=None, locals=None):  # @ReservedAssignment
     if isinstance(code, basestring):
         code = py_compile(code, 'eval')
+
     return py_eval_bytecode(code, globals, locals)
 
 
+def store_eval(code, globals=None, locals=None):
+
+    if globals is None:
+        globals = sys._getframe(1).f_globals
+
+    return py_eval(code, globals, locals)
+
+
 def raise_at_location(e, loc):
     """
     Raises `e` (which must be an Exception object) at location `loc`.
@@ -1841,7 +1998,22 @@ def method_pickle(method):
 def method_unpickle(obj, name):
     return getattr(obj, name)
 
+# Code for pickling modules.
+
+
+def module_pickle(module):
+    if renpy.config.developer:
+        raise Exception("Could not pickle {!r}.".format(module))
+
+    return module_unpickle, (module.__name__,)
+
+
+def module_unpickle(name):
+    return __import__(name)
+
 
 import copy_reg
 import types
-copy_reg.pickle(types.MethodType, method_pickle, method_unpickle)
+
+copy_reg.pickle(types.MethodType, method_pickle)
+copy_reg.pickle(types.ModuleType, module_pickle)
